#include "LCD_3w.h"
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <math.h>
#include <avr/iom8.h>


//max for _delay_ms(14)
//max for _delay_us(48)
//write char to LCD CGRAM

void LCDwritebyte(uint8_t a, uint8_t pc)
{
LCDsendCommand(a);
LCDsendChar(pc);
}
void LCDdefinechar(const uint8_t *pc,uint8_t char_code){
	uint8_t a, pcc;
	uint16_t i;
	a=(char_code<<3)|0x40;
	for (i=0; i<8; i++){
		pcc=pgm_read_byte(&pc[i]);
		LCDwritebyte(a++,pcc);
		}
}
void LCDclr(void)
{
// clear LCD
	LCDsendCommand(1<<LCD_CLR);
}

void LCDhome(void)
{
	// move cursor to home
	LCDsendCommand(1<<LCD_HOME);
}
void sendByteToRegister(uint8_t LCDdata)
{
uint8_t i, temp;
PORTC&=~_BV(datapin);			// sets datapin to output a LOW
for (i=0;i<=7;i++) 				//clear shift 74HC164 register
	{
		PORTC|=_BV(clockpin);		// sets clockpin to output a HIGH
		//_delay_ms(1);
		PORTC&= ~_BV(clockpin);	// sets clockpin to output a LOW
		//_delay_us(40);
	}
temp=LCDdata;	
for (i=0;i<=7;i++)			//write 8bit LCDdata to 74HC164 register
	{
		PORTC |= (LCDdata&1);
		//_delay_us(20);
		PORTC |= _BV(clockpin);		// sets clockpin to output a HIGH
		//_delay_ms(1);
		PORTC&= ~_BV(clockpin);	// sets clockpin to output a LOW
		//_delay_us(40);
		PORTC&=~_BV(datapin);	// sets datapin to output a LOW
		//_delay_us(40);
		LCDdata=temp>>1;
		temp=LCDdata;	
	}
}
void LCDenableCommand()
{
PORTC |= _BV(Epin);
}
void LCDdisableCommand()
{
PORTC &= ~_BV(Epin);
}
void LCDenableData()
{
PORTC |= _BV(datapin);
PORTC |= _BV(Epin);
}
void LCDdisableData()
{
PORTC &= ~_BV(Epin);
PORTC &= ~_BV(datapin);
}
void LCDsendChar(uint8_t letter) //forms data ready to send to 74HC164
{
sendByteToRegister(letter);//sends char to shift register
LCDenableData();
_delay_ms(1);
LCDdisableData();
}
void LCDsendCommand(uint8_t cmd) //forms data ready to send to 74HC164
{
sendByteToRegister(cmd);//sends command to shift register
LCDenableCommand();
_delay_ms(1);
LCDdisableCommand();
}

void LCDstring(uint8_t* data, uint8_t nBytes)
{
	register uint8_t i;

	// check to make sure we have a good pointer
	if (!data) return;

	// print data
	for(i=0; i<nBytes; i++)
	{
		LCDsendChar(data[i]);
	}
}

void LCDGotoXY(uint8_t x, uint8_t y)
{
	register uint8_t DDRAMAddr;

	// remap lines into proper order
	switch(y)
	{
	case 0: DDRAMAddr = LCD_LINE0_DDRAMADDR+x; break;
	case 1: DDRAMAddr = LCD_LINE1_DDRAMADDR+x; break;
	case 2: DDRAMAddr = LCD_LINE2_DDRAMADDR+x; break;
	case 3: DDRAMAddr = LCD_LINE3_DDRAMADDR+x; break;
	default: DDRAMAddr = LCD_LINE0_DDRAMADDR+x;
	}

	// set data address
	LCDsendCommand(1<<LCD_DDRAM | DDRAMAddr);
}



void LCDinit(void)
{
PORTC&=~((1<<datapin)|(1<<clockpin)|(1<<Epin));//set outputs to zerro
DDRC|=(1<<datapin)|(1<<clockpin)|(1<<Epin); //enable output pins
_delay_ms(5); //wait for poverup
sendByteToRegister(0x30);//1
LCDenableCommand();
_delay_ms(1);
LCDdisableCommand();
sendByteToRegister(0x30);//2
LCDenableCommand();
_delay_ms(1);
LCDdisableCommand();
sendByteToRegister(0x30);//3
LCDenableCommand();
_delay_ms(1);
LCDdisableCommand();
_delay_ms(1);//wait for more
sendByteToRegister(0x38);//enable 8 bit mode dual line
LCDenableCommand();
_delay_ms(1);
LCDdisableCommand();
sendByteToRegister(0x0E);// increment adress counter. Cursor shift
LCDenableCommand();
_delay_ms(1);
LCDdisableCommand();
}
