#ifndef LCD_3w_H
#define LCD_3w_H

#include <inttypes.h>

#define datapin     0 	// define datapin from atmega8 to 74HC164
#define clockpin    1 	// define clock pin from atmega8 to 74HC164
#define Epin		2	// controllerpin that enables LCD
#define LCD_CLR             0      // DB0: clear display
#define LCD_HOME            1      // DB1: return to home position
#define LCD_ENTRY_MODE      2      // DB2: set entry mode
#define LCD_ENTRY_INC       1      //   DB1: increment
#define LCD_ENTRY_SHIFT     0      //   DB2: shift
#define LCD_ON_CTRL         3      // DB3: turn lcd/cursor on
#define LCD_ON_DISPLAY      2      //   DB2: turn display on
#define LCD_ON_CURSOR       1      //   DB1: turn cursor on
#define LCD_ON_BLINK        0      //   DB0: blinking cursor
#define LCD_MOVE            4      // DB4: move cursor/display
#define LCD_MOVE_DISP       3      //   DB3: move display (0-> move cursor)
#define LCD_MOVE_RIGHT      2      //   DB2: move right (0-> left)
#define LCD_FUNCTION        5      // DB5: function set
#define LCD_FUNCTION_8BIT   4      //   DB4: set 8BIT mode (0->4BIT mode)
#define LCD_FUNCTION_2LINES 3      //   DB3: two lines (0->one line)
#define LCD_FUNCTION_10DOTS 2      //   DB2: 5x10 font (0->5x7 font)
#define LCD_CGRAM           6      // DB6: set CG RAM address
#define LCD_DDRAM           7      // DB7: set DD RAM address
// reading:
#define LCD_BUSY            7      // DB7: LCD is busy
#define LCD_LINES				2	// visible lines
#define LCD_LINE_LENGTH			16	// line length (in characters)
// cursor position to DDRAM mapping
#define LCD_LINE0_DDRAMADDR		0x00
#define LCD_LINE1_DDRAMADDR		0x40
#define LCD_LINE2_DDRAMADDR		0x14
#define LCD_LINE3_DDRAMADDR		0x54


/*
LCD pins connected to 74hc164:
Q7 ->LCD_D0
Q6 ->LCD_D1
Q5 ->LCD_D2
Q4 ->LCD_D3
Q3 ->LCD_D4
Q2 ->LCD_D5
Q1 ->LCD_D6
Q0 ->LCD_D7
*/



void sendByteToRegister(uint8_t);
void LCDenableCommand(void);
void LCDdisableCommand(void);
void LCDenableData(void);
void LCDdisableData(void);
void LCDsendChar(uint8_t); //forms data ready to send to 74HC164
void LCDsendCommand(uint8_t); //forms data ready to send to 74HC164
void LCDinit(void);
// write a byte to the LCD character generator or display RAM
void LCDwritebyte(uint8_t, uint8_t);
void LCDdefinechar(const uint8_t* ,uint8_t);
void LCDclr(void);
void LCDhome(void);
void LCDstring(uint8_t*, uint8_t);

void LCDGotoXY(uint8_t, uint8_t);




#endif

